/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.bo;

import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spagobi.commons.dao.DAOFactory;
import it.eng.spagobi.commons.utilities.DataSourceUtilities;

import java.sql.Connection;

import org.apache.log4j.Logger;

public class InterfaceType {
    
    private static Logger logger = Logger
    .getLogger(InterfaceType.class);

	private Integer id;
	private String name;
	private String description;
	private String tableName;
	private Integer dataSourceId;
	private String dataSourceLabel;
	private String tableHandlerEntry;
	private String filterClass;

	public Integer getId() {
		return id;
	}

	public void setId(final Integer id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(final String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(final String description) {
		this.description = description;
	}

	public String getTableName() {
		return tableName;
	}

	public void setTableName(final String tableName) {
		this.tableName = tableName;
	}

	public Integer getDataSourceId() {
		return dataSourceId;
	}

	public void setDataSourceId(final Integer dataSourceId) {
		this.dataSourceId = dataSourceId;
		if (dataSourceId != null){
			try {
				this.dataSourceLabel = DAOFactory.getDataSourceDAO()
						.loadDataSourceByID(dataSourceId).getLabel();
				Integer dialectId = DAOFactory.getDataSourceDAO()
				.loadDataSourceByID(dataSourceId).getDialectId();
				this.tableHandlerEntry = DAOFactory.getDomainDAO().loadDomainById(dialectId).getValueCd();
			} catch (EMFUserError e) {
			    logger.error(e.getLocalizedMessage());
			}
		}
	}

	public Connection getConnection(IEngUserProfile userProfile) {
		return new DataSourceUtilities().getConnection(userProfile , dataSourceLabel);
	}

	public DataConnection getDataConnection(IEngUserProfile userProfile) {
		DataConnection toReturn = null;
			try {
				toReturn = new DataSourceUtilities()
						.getDataConnection(getConnection(userProfile));
			} catch (EMFInternalError e) {
			    logger.error(e.getLocalizedMessage());
			}
		return toReturn;
	}

	public String getTableHandlerEntry() {
		return tableHandlerEntry;
	}

	/**
	 * @param filterClass the filterClass to set
	 */
	public void setFilterClass(String filterClass) {
		this.filterClass = filterClass;
	}

	/**
	 * @return the filterClass
	 */
	public String getFilterClass() {
		return filterClass;
	}

}
