/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.tablehandlers.oracle;

import java.util.Date;
import java.util.List;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.extractors.tablehandlers.AbstractTableHandler;
import it.eng.spago4q.extractors.tablehandlers.SqlCodeGeneratorInterface;

public class OracleSqlTableHandler extends AbstractTableHandler {
    
    private SqlCodeGeneratorInterface codeGenerator;
    private String alterSessionQuery;

    public OracleSqlTableHandler() {
        codeGenerator = new OracleSqlCodeGenerator();
        alterSessionQuery = OracleSqlCodeGenerator.getAlterSessionQuery();
    }

    @Override
    public void createFactTable(final DataConnection dataConnection,
            final String tableName, final List interfaceFieldList)
            throws EMFUserError {
        // sequence
        String sequenceQuery = OracleSqlCodeGenerator
                .generateCreateSequenceQuery(tableName);
        // table
        String createQuery = getSqlCodeGenerator().generateCreateTableQuery(
                tableName, interfaceFieldList);
        // trigger
        String triggerQuery = OracleSqlCodeGenerator
                .generateCreateTriggerQuery(tableName);
        String[] queryArray = { sequenceQuery, createQuery, triggerQuery };
        
        queryListExecutor(dataConnection, queryArray);
    }

    @Override
    public void dropFactTable(final DataConnection connection,
            final String tableName) throws EMFUserError {
        // sequence
        String dropSequenceQuery = OracleSqlCodeGenerator
                .generateDropSequenceQuery(tableName);
        // table
        String dropQuery = getSqlCodeGenerator().generateDropTableQuery(
                tableName);
        // trigger
        String dropTrigger = OracleSqlCodeGenerator
                .generateDropTriggerQuery(tableName);
        String[] queryArray = { dropTrigger, dropSequenceQuery, dropQuery };
        
        queryListExecutor(connection, queryArray);
    }

    @Override
    public SqlCodeGeneratorInterface getSqlCodeGenerator() {
        return codeGenerator;
    }

    @Override
    protected SourceBean selectQueryExecutor(
            final DataConnection dataConnection, final String query)
            throws EMFUserError {
        SourceBean result = null;
        SQLCommand sqlCommand = null;
        DataResult dataResult = null;
        try {
            sqlCommand = dataConnection.createInsertCommand(alterSessionQuery);
            sqlCommand.execute();
            sqlCommand = dataConnection.createSelectCommand(query);
            dataResult = sqlCommand.execute();
            ScrollableDataResult scrollableDataResult = (ScrollableDataResult) dataResult
                    .getDataObject();
            result = scrollableDataResult.getSourceBean();
        } catch (EMFInternalError e) {
            throw new EMFUserError(e.getCategory(), e.getDescription());
        } finally {
            Utils.releaseResources(dataConnection, sqlCommand, dataResult);
        }
        return result;
    }

    @Override
    public void updateFactTable(Date executionDate,
            DataConnection dataConnection, String tableName,
            InterfaceItem interfaceItem) throws EMFUserError {
        String updateQuery = getSqlCodeGenerator().generateUpdateQuery(
                executionDate, tableName, interfaceItem);
        String[] queryArray = { alterSessionQuery, updateQuery };

        queryListExecutor(dataConnection, queryArray);
    }

    @Override
    public void insertIntoFactTable(Date executionDate,
            DataConnection dataConnection, String tableName,
            InterfaceItem interfaceItem, Integer logId) throws EMFUserError {
        String insertQuery = getSqlCodeGenerator().generateInsertQuery(
                executionDate, tableName, interfaceItem, logId);
        String[] queryArray = { alterSessionQuery, insertQuery };
        
        queryListExecutor(dataConnection, queryArray);
    }

}
